@@ -37,41 +37,42 @@ class View
 #     end
   end
 
+  def difference_whole()
+    @difference
+  end
+
+  def difference_digest()
+    #
+  end
+
+  def escape_inside(str, tags)
+    str.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]}
+  end
+  def escape_outside(str, tags)
+    str.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]}
+  end
+
   def apply_style(tags, headfoot = true)
     result = []
     @difference.each{|block|
       operation = block.first
       if block_given?
-        source = yield block[1].to_s
-        target = yield block[2].to_s
+        src = yield block[1].to_s
+        tgt = yield block[2].to_s
       else
-        source = block[1].to_s
-        target = block[2].to_s
+        src = block[1].to_s
+        tgt = block[2].to_s
       end
       case operation
       when :common_elt_elt
-        result << (
-          tags[:start_common] +
-          source.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} +
-          tags[:end_common]
-        )
+        result << (tags[:start_common] + escape_outside(src, tags) + tags[:end_common])
       when :change_elt
-        result << (tags[:start_before_change] + 
-          source.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-                   tags[:end_before_change] + 
-                   tags[:start_after_change] + 
-          target.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-                   tags[:end_after_change])
+        result << (tags[:start_before_change] + escape_inside(src, tags) + tags[:end_before_change] +
+                   tags[:start_after_change]  + escape_inside(tgt, tags) + tags[:end_after_change])
       when :del_elt
-        result << (tags[:start_del] +
-          source.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-          tags[:end_del]
-        )
+        result << (tags[:start_del] + escape_inside(src, tags) + tags[:end_del])
       when :add_elt
-        result << (tags[:start_add] +
-          target.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-          tags[:end_add]
-        )
+        result << (tags[:start_add] + escape_inside(tgt, tags) + tags[:end_add])
       else
         raise "invalid attribute: #{block.first}\n"
       end
@@ -81,92 +82,99 @@ class View
     end
     result.delete_if{|elem|elem==''}
   end
+
+  CONTEXT_PRE_LENGTH  = 32
+  CONTEXT_POST_LENGTH = 32
   def apply_style_digest(tags, headfoot = true)
+    cxt_pre_pat  = Regexp.new('.{0,'+"#{CONTEXT_PRE_LENGTH}"+'}\Z',
+                              Regexp::MULTILINE, @encoding.sub(/ASCII/i, 'none'))
+    cxt_post_pat = Regexp.new('\A.{0,'+"#{CONTEXT_POST_LENGTH}"+'}',
+                              Regexp::MULTILINE, @encoding.sub(/ASCII/i, 'none'))
+    display = (tags and tags[:display]) || 'inline'
     result = []
-    doc1_lnum = 1
-    doc2_lnum = 1
-    @difference.each_with_index{|block, i|
-      operation = block.first
+    d1l = doc1_line_number = 1
+    d2l = doc2_line_number = 1
+    @difference.each_with_index{|entry, i|
       if block_given?
-        source = yield block[1].to_s
-        target = yield block[2].to_s
+        src = yield entry[1].to_s
+        tgt = yield entry[2].to_s
       else
-        source = block[1].to_s
-        target = block[2].to_s
+        src = entry[1].to_s
+        tgt = entry[2].to_s
       end
-      span1 = source_lines_involved = source.scan_lines(@eol).size
-      span2 = target_lines_involved = target.scan_lines(@eol).size
-      pos = ""
+      cxt_pre = if i == 0
+                   ""  # no pre context for the first entry
+                else
+                  @difference[i-1][1].to_s.scan(cxt_pre_pat).to_s
+                end
+      cxt_post = if (i + 1) == @difference.size
+                   ""  # no post context for the last entry
+                 else
+                   @difference[i+1][1].to_s.scan(cxt_post_pat).to_s
+                 end
+      # elements for an entry
+      e_head     = Proc.new {|pos_str|
+                             tags[:start_entry] + tags[:start_position] + pos_str + tags[:end_position]}
+      e_cxt_pre  = tags[:start_prefix] + escape_outside(cxt_pre, tags) + tags[:end_prefix]
+      e_src      = escape_inside(src, tags)
+      e_chg      = tags[:start_before_change] + escape_inside(src, tags)  + tags[:end_before_change] +
+                   tags[:start_after_change]  + escape_inside(tgt, tags)  + tags[:end_after_change]
+      e_chgdel   = tags[:start_before_change] + escape_inside(src, tags)  + tags[:end_before_change]
+      e_chgadd   = tags[:start_after_change]  + escape_inside(tgt, tags)  + tags[:end_after_change]
+      e_del      = tags[:start_del]           + escape_outside(src, tags) + tags[:end_del]
+      e_add      = tags[:start_add]           + escape_outside(tgt, tags) + tags[:end_add]
+      e_cxt_post = tags[:start_postfix]       + escape_outside(cxt_post, tags) + tags[:end_postfix]
+      e_foot     = tags[:end_entry] + (@eol_char||"")
 
-      case
-      when i == 0 then prefix = ""
-      else prefix = @difference[i-1][1].to_s.scan(prefix_pat).to_s
-      end
-      case
-      when (i + 1) == @difference.size then postfix = ""
-      else postfix = @difference[i+1][1].to_s.scan(postfix_pat).to_s
-      end
-
-      case operation
+      span1 = source_lines_involved = src.scan_lines(@eol).size
+      span2 = target_lines_involved = tgt.scan_lines(@eol).size
+      pos_str = ""
+      case operation = entry.first
       when :common_elt_elt
+        # skipping common part
       when :change_elt
-        pos += "#{doc1_lnum}"
-        pos += "-#{doc1_lnum + span1 - 1}" if span1 > 1
-        pos += ",#{doc2_lnum}"
-        pos += "-#{doc2_lnum + span2 - 1}" if span2 > 1
-        result << (
-          tags[:start_entry] + 
-          tags[:start_position] + pos + tags[:end_position] + 
-          tags[:start_prefix] + prefix.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} + tags[:end_prefix] +
-          tags[:start_before_change] +
-          source.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-          tags[:end_before_change] + 
-          tags[:start_after_change] +
-          target.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-          tags[:end_after_change] +
-          tags[:start_postfix] + postfix.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} + tags[:end_postfix] +
-          tags[:end_entry] + (@eol_char||"")
-        )
+        pos_str = "#{d1l}" + "#{if span1 > 1 then '-'+(d1l + span1 - 1).to_s; end}" +
+                  ",#{d2l}" + "#{if span2 > 1 then '-'+(d2l + span2 - 1).to_s; end}"
+        case display
+        when 'inline'
+          result << (e_head.call(pos_str) + e_cxt_pre + e_chg + e_cxt_post + e_foot)
+        when 'multi'
+          result << (e_head.call(pos_str) + e_cxt_pre + e_chgdel + e_cxt_post +
+                                            e_cxt_pre + e_chgadd + e_cxt_post + e_foot)
+        else raise "Unsupported display type: #{display}"
+        end
       when :del_elt
-        pos += "#{doc1_lnum}"
-        pos += "-#{doc1_lnum + span1 - 1}" if span1 > 1
-        pos += ",(#{doc2_lnum})"
-        result << (
-          tags[:start_entry] +
-          tags[:start_position] + pos + tags[:end_position] +
-          tags[:start_prefix] + prefix.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} + tags[:end_prefix] +
-          tags[:start_del] +
-          source.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-          tags[:end_del] + 
-          tags[:start_postfix] + postfix.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} + tags[:end_postfix] +
-          tags[:end_entry] + (@eol_char||"")
-        )
+        pos_str = "#{d1l}" + "#{if span1 > 1 then '-'+(d1l + span1 - 1).to_s; end}" +
+                  ",(#{d2l})"
+        case display
+        when 'inline'
+          result << (e_head.call(pos_str) + e_cxt_pre + e_del + e_cxt_post + e_foot)
+        when 'multi'
+          result << (e_head.call(pos_str) + e_cxt_pre + e_src + e_cxt_post +
+                                            e_cxt_pre + e_del + e_cxt_post + e_foot)
+        else raise "Unsupported display type: #{display}"
+        end
       when :add_elt
-        pos += "(#{doc1_lnum})"
-        pos += ",#{doc2_lnum}"
-        pos += "-#{doc2_lnum + span2 - 1}" if span2 > 1
-        result << (
-          tags[:start_entry] +
-          tags[:start_position] + pos + tags[:end_position] +
-          tags[:start_prefix] + prefix.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} + tags[:end_prefix] +
-          tags[:start_add] +
-          target.gsub(tags[:inside_escape_pat]){|m| tags[:inside_escape_dic][m]} +
-          tags[:end_add] +
-          tags[:start_postfix] + postfix.gsub(tags[:outside_escape_pat]){|m| tags[:outside_escape_dic][m]} + tags[:end_postfix] +
-          tags[:end_entry] + (@eol_char||"")
-        )
+        pos_str = "(#{d1l})" +
+                  ",#{d2l}" + "#{if span2 > 1 then '-'+(d2l + span2 - 1).to_s; end}"
+        case display
+        when 'inline'
+          result << (e_head.call(pos_str) + e_cxt_pre + e_add + e_cxt_post + e_foot)
+        when 'multi'
+          result << (e_head.call(pos_str) + e_cxt_pre + e_src + e_cxt_post +
+                                            e_cxt_pre + e_add + e_cxt_post + e_foot)
+        else raise "Unsupported display type: #{display}"
+        end
       else
         raise "invalid attribute: #{block.first}\n"
       end
-      doc1_lnum += source.scan_eols(@eol).size
-      doc2_lnum += target.scan_eols(@eol).size
+      d1l += src.scan_eols(@eol).size
+      d2l += tgt.scan_eols(@eol).size
     }
     result.unshift(tags[:start_digest_body])
     result.push(tags[:end_digest_body])
-    if headfoot == true
-      result = tags[:header] + result + tags[:footer]
-    end
-    result.delete_if{|elem|elem==''}
+    result = tags[:header] + result + tags[:footer] if headfoot == true
+    result.delete_if{|elem| elem == ''}
   end
 
   def source_lines()
@@ -181,16 +189,6 @@ class View
     end
     @target_lines
   end
-  PREFIX_LENGTH = 16
-  POSTFIX_LENGTH = 16
-  def prefix_pat()
-    Regexp.new('.{0,'+"#{PREFIX_LENGTH}"+'}\Z', Regexp::MULTILINE,
-               @encoding.sub(/ASCII/i, 'none'))
-  end
-  def postfix_pat()
-    Regexp.new('\A.{0,'+"#{POSTFIX_LENGTH}"+'}', Regexp::MULTILINE,
-               @encoding.sub(/ASCII/i, 'none'))
-  end
 
   # tty (terminal)
   def tty_header()
@@ -209,13 +207,13 @@ class View
      :start_digest_body   => "----#{@eol_char||''}",
      :end_digest_body     => '',
      :start_entry         => '',
-     :end_entry           => "#{@eol_char||''}----",
+     :end_entry           => "----",
      :start_position      => '',
      :end_position        => "#{@eol_char||''}",
      :start_prefix        => '',
      :end_prefix          => '',
      :start_postfix       => '',
-     :end_postfix         => '',
+     :end_postfix         => "#{@eol_char||''}",
      :header              => tty_header(),
      :footer              => tty_footer(),
      :start_common        => '',
@@ -229,14 +227,14 @@ class View
      :start_after_change  => "\033[7;1;32m",  # Inverted, Bold, Green
      :end_after_change    => "\033[0m"}
   end
-  def to_tty(overriding_tags = nil, headfoot = true)  # color escape sequence
+  def to_tty(overriding_opts = nil, headfoot = true)  # color escape sequence
     tags = tty_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style(tags, headfoot)
   end
-  def to_tty_digest(overriding_tags = nil, headfoot = true)
+  def to_tty_digest(overriding_opts = nil, headfoot = true)
     tags = tty_tags
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style_digest(tags, headfoot)
   end
 
@@ -252,15 +250,16 @@ class View
      " body {font-family: monospace;}#{@eol_char||''}" +
      " span.del {background: hotpink; border: thin inset;}#{@eol_char||''}" +
      " span.add {background: deepskyblue; font-weight: bolder; border: thin outset;}#{@eol_char||''}" +
-     " span.before_change {background: yellow; border: thin inset;}#{@eol_char||''}" +
-     " span.after_change {background: lime; font-weight: bolder; border: thin outset;}#{@eol_char||''}" +
-     " li.entry .position {font-weight: bolder; margin-top: 0em; margin-bottom: 0em; padding-top: 0em; padding-bottom: 0em;}#{@eol_char||''}" +
-     " li.entry .body {margin-top: 0em; margin-bottom: 0em; padding-top: 0em; padding-bottom: 0em;}#{@eol_char||''}" +
+     " span.before-change {background: yellow; border: thin inset;}#{@eol_char||''}" +
+     " span.after-change {background: lime; font-weight: bolder; border: thin outset;}#{@eol_char||''}" +
+     " li.entry .position {font-weight: bolder; margin-top: 0em; margin-bottom: 0em; padding-top: 0.5em; padding-bottom: 0em;}#{@eol_char||''}" +
+     " li.entry .body {margin-top: 0em; margin-bottom: 0em; padding-top: 0em; padding-bottom: 0.5em;}#{@eol_char||''}" +
+     " li.entry {border-top: thin solid gray;}#{@eol_char||''}" +
      "</style>#{@eol_char||''}",
-     "</head><body>#{@eol_char||''}"]
+     "</head><body><div>#{@eol_char||''}"]
   end
   def html_footer()
-    [(@eol_char||"") + '</body></html>' + (@eol_char||"")]
+    [(@eol_char||"") + '</div></body></html>' + (@eol_char||"")]
   end
   HTMLEscapeDic = {'<'=>'&lt;', '>'=>'&gt;', '&'=>'&amp;', '  '=>'&nbsp;&nbsp;',
                    "\r\n" => "<br />\r\n", "\r" => "<br />\r", "\n" => "<br />\n"}
@@ -270,16 +269,16 @@ class View
      :outside_escape_pat  => HTMLEscapePat,
      :inside_escape_dic   => HTMLEscapeDic,
      :inside_escape_pat   => HTMLEscapePat,
-     :start_digest_body   => '<ul><hr />',
+     :start_digest_body   => '<ul>',
      :end_digest_body     => '</ul>',
      :start_entry         => '<li class="entry">',
-     :end_entry           => '</p></blockquote></li><hr />',
+     :end_entry           => '</blockquote></li>',
      :start_position      => '<p class="position">',
-     :end_position        => '</p><blockquote class="body"><p class="body">',
-     :start_prefix        => '',
+     :end_position        => '</p><blockquote class="body">',
+     :start_prefix        => '<p class="body">',
      :end_prefix          => '',
      :start_postfix       => '',
-     :end_postfix         => '',
+     :end_postfix         => '</p>',
      :header              => html_header(),
      :footer              => html_footer(),
      :start_common        => '<span class="common">',
@@ -288,19 +287,19 @@ class View
      :end_del             => '</del></span>',
      :start_add           => '<span class="add"><ins>',
      :end_add             => '</ins></span>',
-     :start_before_change => '<span class="before_change"><del>',
+     :start_before_change => '<span class="before-change"><del>',
      :end_before_change   => '</del></span>',
-     :start_after_change  => '<span class="after_change"><ins>',
+     :start_after_change  => '<span class="after-change"><ins>',
      :end_after_change    => '</ins></span>'}
   end
-  def to_html(overriding_tags = nil, headfoot = true)
+  def to_html(overriding_opts = nil, headfoot = true)
     tags = html_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style(tags, headfoot)
   end
-  def to_html_digest(overriding_tags = nil, headfoot = true)
+  def to_html_digest(overriding_opts = nil, headfoot = true)
     tags = html_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style_digest(tags, headfoot)
   end
 
@@ -329,13 +328,13 @@ class View
      :start_digest_body   => "----#{@eol_char||''}",
      :end_digest_body     => '',
      :start_entry         => '',
-     :end_entry           => "#{@eol_char||''}----",
+     :end_entry           => "----",
      :start_position      => '',
      :end_position        => "#{@eol_char||''}",
      :start_prefix        => '',
      :end_prefix          => '',
      :start_postfix       => '',
-     :end_postfix         => '',
+     :end_postfix         => "#{@eol_char||''}",
      :header              => manued_header(),
      :footer              => manued_footer(),
      :start_common        => '',
@@ -350,14 +349,19 @@ class View
      :end_after_change    => ']'
     }
   end
-  def to_manued(overriding_tags = nil, headfoot = true)  # [ / ; ]
+  def to_manued(overriding_opts = nil, headfoot = true)  # [ / ; ]
     tags = manued_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style(tags, headfoot)
   end
-  def to_manued_digest(overriding_tags = nil, headfoot = true)  # [ / ; ]
+  def to_manued_digest(overriding_opts = nil, headfoot = true)  # [ / ; ]
     tags = manued_tags()
-    tags.update(overriding_tags) if overriding_tags
+    # manued specific kludge: change should be [a/b] in inline, [a/][/b] in multi
+    display = (overriding_opts and overriding_opts[:display]) || 'inline'
+    if display == 'multi'
+      tags.update({:end_before_change => '/]', :start_after_change => '[/'})
+    end
+    tags.update(overriding_opts) if overriding_opts
     apply_style_digest(tags, headfoot)
   end
 
@@ -378,13 +382,13 @@ class View
      :start_digest_body   => "----#{@eol_char||''}",
      :end_digest_body     => '',
      :start_entry         => '',
-     :end_entry           => "#{@eol_char||''}----",
+     :end_entry           => "----",
      :start_position      => '',
      :end_position        => "#{@eol_char||''}",
      :start_prefix        => '',
      :end_prefix          => '',
      :start_postfix       => '',
-     :end_postfix         => '',
+     :end_postfix         => "#{@eol_char||''}",
      :header              => wdiff_header(),
      :footer              => wdiff_footer(),
      :start_common        => '',
@@ -398,14 +402,14 @@ class View
      :start_after_change  => '{+',
      :end_after_change    => '+}'}
   end
-  def to_wdiff(overriding_tags = nil, headfoot = true)
+  def to_wdiff(overriding_opts = nil, headfoot = true)
     tags = wdiff_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style(tags)
   end
-  def to_wdiff_digest(overriding_tags = nil, headfoot = true)
+  def to_wdiff_digest(overriding_opts = nil, headfoot = true)
     tags = wdiff_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style_digest(tags, headfoot)
   end
 
@@ -442,14 +446,14 @@ class View
      :start_after_change  => '',
      :end_after_change    => ''}
   end
-  def to_user(overriding_tags = nil, headfoot = true)
+  def to_user(overriding_opts = nil, headfoot = true)
     tags = user_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style(tags, headfoot)
   end
-  def to_user_digest(overriding_tags = nil, headfoot = true)
+  def to_user_digest(overriding_opts = nil, headfoot = true)
     tags = user_tags()
-    tags.update(overriding_tags) if overriding_tags
+    tags.update(overriding_opts) if overriding_opts
     apply_style_digest(tags, headfoot)
   end
 
